#pragma option -mt -k- -p- -r- -G- -N-

#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>

//The interrupt no.
#define INTR_33 0x33

//structure to access stored register values in interrupt functions
struct INTR_REGS {
  unsigned bp, di, si, ds, es, dx, cx, bx, ax, ip, cs, fl;
};

int volatile mouseMoveX; //mouse motion counters
int volatile mouseMoveY;
int volatile moveRateX;  //speed of mouse cursor
int volatile moveRateY;

//pointers to code
void interrupt (*oldhandler_33)(void);  //original handler of int 33h

#pragma argsused

void interrupt handler_33(struct INTR_REGS r) {

  //if reading the motion counters
  if(r.ax==0x0b) {
    //call the old interrupt handler to get counters
    asm mov ax,0x0b
    asm pushf
    asm call cs:oldhandler_33
    asm mov mouseMoveX,cx
    asm mov mouseMoveY,dx
    //modify the counters
    mouseMoveX=mouseMoveX*moveRateX/100;
    mouseMoveY=mouseMoveY*moveRateY/100;
    //return the modified counters
    asm mov cx,mouseMoveX
    asm mov dx,mouseMoveY
    //clean up the stack after borland interrupt function
    asm pop bp
    asm pop di
    asm pop si
    asm pop ds
    asm pop es
    asm add sp,4
    asm pop bx
    asm pop ax
    //return from interrupt
    asm iret

  } else {

    //clean up the stack after borland interrupt function
    asm pop bp
    asm pop di
    asm pop si
    asm pop ds
    asm pop es
    asm pop dx
    asm pop cx
    asm pop bx
    asm pop ax
    //transfer control to the old interrupt handler
    asm jmp cs:oldhandler_33
  }
}

int main(int argc, char** argv) {

  printf("MOUSEX - Mouse Acceleration Utility\n");
  printf("Based on MOUSE2KV by Peter Maceluch\n\n");

  if(argc<4) {
    printf("Incorrect number of parameters\n\n"
    "  Usage: MOUSEX xspeed yspeed filename.ext\n\n"
    "  xspeed, yspeed - change to mouse speed as a percentage of normal\n"
    "  filename.ext   - program to launch with mouse acceleration\n\n"
    "  e.g. MOUSEX 150 100 GAME.EXE\n");
    return 1;
  }

  moveRateX=strtol(argv[1],NULL,0);
  moveRateY=strtol(argv[2],NULL,0);

  if(moveRateY==0||moveRateX==0) {
    printf("Incorrect values entered in parameters\n");
    return 2;
  }

  //get the address of the current interrupt handler
  oldhandler_33=getvect(INTR_33);

  //install the new interrupt handler
  setvect(INTR_33,handler_33);

  //execute client program
  spawnvp(P_WAIT,argv[3],&(argv[3]));

  //restore the old interrupt handler
  setvect(INTR_33,oldhandler_33);

  return 0;
}
